#version 430 core

layout(quads, fractional_odd_spacing, ccw) in;

// Input variables
in VertexData
{
	vec3 position_ws;	// Position in world space
	vec3 tangent_ws;	// Tangent in world space
	vec3 bitangent_ws;	// Bitangent in world space
} inData[];

// Output variables
out VertexData
{
	vec3 position_ws;	// Position in world space
	vec3 normal_ws;		// Normal in world space
	vec2 tex_coord;
	vec3 tangent_ws;	// Tangent in world space
	vec3 bitangent_ws;	// Bitangent in world space
} outData;

// Data of the camera
layout (std140, binding = 0) uniform CameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};

// Height texture and a scale applied to it
layout (binding = 2) uniform sampler2D height_tex;
uniform float height_scale;

// Amount of displacement, and whether to use it or not
const float max_displacement = 0.1f;
uniform bool use_displacement_mapping;

// Evaluates cubic bezier spline
vec3 bezier4(in vec3 v0, in vec3 v1, in vec3 v2, in vec3 v3, in float t)
{
	// Task 1: Evaluate cubic bezier spline
	//		Hint: Precompute (1-t) and store it into a variable
	return v1;
}

// Evaluates cubic bezier patch
vec3 bezier4x4(
	in vec3 v0,  in vec3 v1,  in vec3 v2,  in vec3 v3,
	in vec3 v4,  in vec3 v5,  in vec3 v6,  in vec3 v7,
	in vec3 v8,  in vec3 v9,  in vec3 v10, in vec3 v11,
	in vec3 v12, in vec3 v13, in vec3 v14, in vec3 v15, in vec2 t)
{
	// Task 1: Evaluate cubic bezier patch
	//		Hint: Use bezier4 function, use t.x and t.y
	return v1;
}

//----------------------------------------------------------------

void main()
{
	// Task 1: Evaluate the position of the vertex in the Bezier patch.
	//			Store the result into outData.position_ws
	//		Hint: Implement functions bezier4 and bezier4x4. Patch coordinate is in gl_TessCoord.xy.
	outData.position_ws = vec3(0.0);

	// Task 2: Compute the texture coordinates, use the patch coordinates as texture coordinates.
	//			Store the result into outData.tex_coord

	// Task 3: Evaluate the tangent and bitangent of the vertex in the Bezier patch.
	//			Store the result into outData.tangent_ws/bitangent_ws
	//		Hint: Use the same functions as for the position

	// Task 3: Compute the normal as the cross product of the tangent and bitangent, and normalize it.
	//			Store the result into outData.normal_ws

	if (use_displacement_mapping)
	{
		// Task 5: Displace the vertex position
		//		Hint: function textureLod, texture height_tex, constant max_displacement, uniform height_scale
	}

	// Compute gl_Position
	gl_Position = projection * view * vec4(outData.position_ws, 1.0);
}